/* eslint no-console: 0 */
/* eslint global-require: 0 */
/* eslint import/no-dynamic-require: 0 */

// Fetch NODE_ENV or setup default to production
process.env.NODE_ENV = process.env.NODE_ENV || 'development'

const chalk = require('chalk')
const detect = require('detect-port')
const Webpack = require('webpack')
const { resolve } = require('path')
const { safeLoad } = require('js-yaml')
const { readFileSync } = require('fs')
const WebpackDevServer = require('webpack-dev-server')

const configPath = resolve(process.cwd(), 'config', 'webpacker.yml')
const { paths, devServer } = safeLoad(readFileSync(configPath), 'utf8')[process.env.NODE_ENV]
const webpackConfig = require(resolve(process.cwd(), paths.config, 'development.js'))
const logErrors = require('../utils/log-errors')

// Setup compiler with entries
const compiler = Webpack(webpackConfig, (err, stats) => {
  const info = stats.toJson()

  if (err) {
    console.log(`${chalk.blue('[webpacker]')} ${chalk.red('Failed to compile.')}`)
    console.error(`${chalk.blue('[webpacker]')} ${chalk.red(err.stack || err)}`)
    console.log(`${chalk.blue('[webpacker]')} ${chalk.red(err.details)}`)
    return
  }

  if (stats.hasErrors()) {
    console.log(`${chalk.blue('[webpacker]')} ${chalk.red('Failed to compile.')}`)
    logErrors(info.errors)
    return
  }

  if (stats.hasWarnings()) {
    console.log(`${chalk.blue('[webpacker]')} ${chalk.yellow('Compiled with warnings.')}`)
    logErrors(info.warnings)
  }

  console.log(`${chalk.blue('[webpacker]')} ${chalk.green('Compiled successfully!')}`)
  console.log()
  console.log(`${chalk.blue('[webpacker]')} The webpack-dev-server is serving assets from:`)
  console.log(`${chalk.blue('[webpacker]')} http${devServer.https ? 's' : ''}://${devServer.host}:${devServer.port}/${paths.output}`)
  console.log()
})

const run = () => {
  // Initialize webpack-dev-server with compiler and dev server options
  const webpackDevServer = new WebpackDevServer(compiler, webpackConfig.devServer)

  // Launch webpack-dev-server on configured port and host
  webpackDevServer.listen(devServer.port, (err) => {
    if (err) { console.log(chalk.red(err)) }
    console.log(`${chalk.blue('[webpacker]')} ${chalk.cyan('Starting the webpack dev server...')}`)
    console.log()
  })
}

// Check port and call function to run webpack-dev-server
detect(devServer.port).then((port) => {
  if (devServer.port === port) {
    run()
  } else {
    console.log(`${chalk.blue('[webpacker]')} ${chalk.red(`Another application is using port ${devServer.port}.`)}`)
    process.exit(1)
  }
})
